package dev.shadowsoffire.apotheosis.ench.library;

package .tmp;

import dev.shadowsoffire.apotheosis.ench.library.EnchLibraryContainer;
import dev.shadowsoffire.placebo.recipe.VanillaPacketDispatcher;
import it.unimi.dsi.fastutil.objects.Object2IntMap;
import it.unimi.dsi.fastutil.objects.Object2IntOpenHashMap;
import it.unimi.dsi.fastutil.objects.ObjectIterator;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.nbt.CompoundTag;
import net.minecraft.nbt.Tag;
import net.minecraft.network.Connection;
import net.minecraft.network.protocol.Packet;
import net.minecraft.network.protocol.game.ClientboundBlockEntityDataPacket;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.item.ItemStack;
import net.minecraft.world.item.Items;
import net.minecraft.world.item.enchantment.Enchantment;
import net.minecraft.world.item.enchantment.EnchantmentHelper;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraftforge.common.capabilities.Capability;
import net.minecraftforge.common.capabilities.ForgeCapabilities;
import net.minecraftforge.common.util.LazyOptional;
import net.minecraftforge.items.IItemHandler;
import net.minecraftforge.registries.ForgeRegistries;

public abstract class EnchLibraryTile extends BlockEntity {
  protected final Object2IntMap<Enchantment> points = (Object2IntMap<Enchantment>)new Object2IntOpenHashMap();
  
  protected final Object2IntMap<Enchantment> maxLevels = (Object2IntMap<Enchantment>)new Object2IntOpenHashMap();
  
  protected final Set<EnchLibraryContainer> activeContainers = new HashSet<>();
  
  protected final LazyOptional<IItemHandler> itemHandler = LazyOptional.of(() -> new EnchLibItemHandler(this));
  
  protected final int maxLevel;
  
  protected final int maxPoints;
  
  public EnchLibraryTile(BlockEntityType<?> type, BlockPos pos, BlockState state, int maxLevel) {
    super(type, pos, state);
    this.maxLevel = maxLevel;
    this.maxPoints = levelToPoints(maxLevel);
  }
  
  public void depositBook(ItemStack book) {
    if (book.m_41720_() != Items.f_42690_)
      return; 
    Map<Enchantment, Integer> enchs = EnchantmentHelper.m_44831_(book);
    for (Map.Entry<Enchantment, Integer> e : enchs.entrySet()) {
      if (e.getKey() == null || e.getValue() == null)
        continue; 
      int newPoints = Math.min(this.maxPoints, this.points.getInt(e.getKey()) + levelToPoints(((Integer)e.getValue()).intValue()));
      if (newPoints < 0)
        newPoints = this.maxPoints; 
      this.points.put(e.getKey(), newPoints);
      this.maxLevels.put(e.getKey(), Math.min(this.maxLevel, Math.max(this.maxLevels.getInt(e.getKey()), ((Integer)e.getValue()).intValue())));
    } 
    if (enchs.size() > 0)
      VanillaPacketDispatcher.dispatchTEToNearbyPlayers(this); 
    m_6596_();
  }
  
  public void extractEnchant(ItemStack stack, Enchantment ench, int level) {
    int curLvl = ((Integer)EnchantmentHelper.m_44831_(stack).getOrDefault(ench, Integer.valueOf(0))).intValue();
    if (stack.m_41619_() || !canExtract(ench, level, curLvl) || level == curLvl)
      return; 
    Map<Enchantment, Integer> enchs = EnchantmentHelper.m_44831_(stack);
    enchs.put(ench, Integer.valueOf(level));
    EnchantmentHelper.m_44865_(enchs, stack);
    this.points.put(ench, Math.max(0, this.points.getInt(ench) - levelToPoints(level) + levelToPoints(curLvl)));
    if (!this.f_58857_.m_5776_())
      VanillaPacketDispatcher.dispatchTEToNearbyPlayers(this); 
    m_6596_();
  }
  
  public boolean canExtract(Enchantment ench, int level, int currentLevel) {
    return (this.maxLevels.getInt(ench) >= level && this.points.getInt(ench) >= levelToPoints(level) - levelToPoints(currentLevel));
  }
  
  public static int levelToPoints(int level) {
    return (int)Math.pow(2.0D, (level - 1));
  }
  
  public void m_183515_(CompoundTag tag) {
    CompoundTag points = new CompoundTag();
    for (ObjectIterator<Object2IntMap.Entry<Enchantment>> objectIterator1 = this.points.object2IntEntrySet().iterator(); objectIterator1.hasNext(); ) {
      Object2IntMap.Entry<Enchantment> e = objectIterator1.next();
      points.m_128405_(ForgeRegistries.ENCHANTMENTS.getKey(e.getKey()).toString(), e.getIntValue());
    } 
    tag.m_128365_("Points", (Tag)points);
    CompoundTag levels = new CompoundTag();
    for (ObjectIterator<Object2IntMap.Entry<Enchantment>> objectIterator2 = this.maxLevels.object2IntEntrySet().iterator(); objectIterator2.hasNext(); ) {
      Object2IntMap.Entry<Enchantment> e = objectIterator2.next();
      levels.m_128405_(ForgeRegistries.ENCHANTMENTS.getKey(e.getKey()).toString(), e.getIntValue());
    } 
    tag.m_128365_("Levels", (Tag)levels);
    super.m_183515_(tag);
  }
  
  public void m_142466_(CompoundTag tag) {
    super.m_142466_(tag);
    CompoundTag points = tag.m_128469_("Points");
    for (String s : points.m_128431_()) {
      Enchantment ench = (Enchantment)ForgeRegistries.ENCHANTMENTS.getValue(new ResourceLocation(s));
      if (ench == null)
        continue; 
      this.points.put(ench, points.m_128451_(s));
    } 
    CompoundTag levels = tag.m_128469_("Levels");
    for (String s : levels.m_128431_()) {
      Enchantment ench = (Enchantment)ForgeRegistries.ENCHANTMENTS.getValue(new ResourceLocation(s));
      if (ench == null)
        continue; 
      this.maxLevels.put(ench, levels.m_128451_(s));
    } 
  }
  
  public void onDataPacket(Connection net, ClientboundBlockEntityDataPacket pkt) {
    CompoundTag tag = pkt.m_131708_();
    CompoundTag points = tag.m_128469_("Points");
    for (String s : points.m_128431_()) {
      Enchantment ench = (Enchantment)ForgeRegistries.ENCHANTMENTS.getValue(new ResourceLocation(s));
      if (ench == null)
        continue; 
      this.points.put(ench, points.m_128451_(s));
    } 
    CompoundTag levels = tag.m_128469_("Levels");
    for (String s : levels.m_128431_()) {
      Enchantment ench = (Enchantment)ForgeRegistries.ENCHANTMENTS.getValue(new ResourceLocation(s));
      if (ench == null)
        continue; 
      this.maxLevels.put(ench, levels.m_128451_(s));
    } 
    this.activeContainers.forEach(EnchLibraryContainer::onChanged);
  }
  
  public ClientboundBlockEntityDataPacket getUpdatePacket() {
    return ClientboundBlockEntityDataPacket.m_195640_(this);
  }
  
  public CompoundTag m_5995_() {
    CompoundTag tag = super.m_5995_();
    CompoundTag points = new CompoundTag();
    for (ObjectIterator<Object2IntMap.Entry<Enchantment>> objectIterator1 = this.points.object2IntEntrySet().iterator(); objectIterator1.hasNext(); ) {
      Object2IntMap.Entry<Enchantment> e = objectIterator1.next();
      points.m_128405_(ForgeRegistries.ENCHANTMENTS.getKey(e.getKey()).toString(), e.getIntValue());
    } 
    tag.m_128365_("Points", (Tag)points);
    CompoundTag levels = new CompoundTag();
    for (ObjectIterator<Object2IntMap.Entry<Enchantment>> objectIterator2 = this.maxLevels.object2IntEntrySet().iterator(); objectIterator2.hasNext(); ) {
      Object2IntMap.Entry<Enchantment> e = objectIterator2.next();
      levels.m_128405_(ForgeRegistries.ENCHANTMENTS.getKey(e.getKey()).toString(), e.getIntValue());
    } 
    tag.m_128365_("Levels", (Tag)levels);
    return tag;
  }
  
  public Object2IntMap<Enchantment> getPointsMap() {
    return this.points;
  }
  
  public Object2IntMap<Enchantment> getLevelsMap() {
    return this.maxLevels;
  }
  
  public void addListener(EnchLibraryContainer ctr) {
    this.activeContainers.add(ctr);
  }
  
  public void removeListener(EnchLibraryContainer ctr) {
    this.activeContainers.remove(ctr);
  }
  
  public int getMax(Enchantment ench) {
    return Math.min(this.maxLevel, this.maxLevels.getInt(ench));
  }
  
  public <T> LazyOptional<T> getCapability(Capability<T> cap, Direction side) {
    if (cap == ForgeCapabilities.ITEM_HANDLER)
      return this.itemHandler.cast(); 
    return super.getCapability(cap, side);
  }
}
